/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci <dark.schneider@iol.it>
    Based on a code from Luigi Quartapelle and Massimo Biava

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Created on March 8, 2010, 7:25 PM
 */

#ifndef _U2D_HELMHOLTZ_H
#define	_U2D_HELMHOLTZ_H

#include "boost/numeric/mtl/mtl.hpp"
#include "u2d_Mesh.h"
#include "u2d_load_umesh2d.h"
#include "u2d_IsoElement2DP13Points.h"
#include "u2d_IsoElement1DP12Points.h"
#include "u2d_DirichletCondition.h"
#include "u2d_NeumannCondition.h"
#include "u2d_Problem.h"
#include "u2d_ProblemCalculator.h"
#include "common/u2d_Project.h"

using namespace mtl;

class u2d_Helmholtz {
public:
    u2d_Helmholtz(u2d_Project *project);
    u2d_Helmholtz(string meshdir, string problemname);
    u2d_Helmholtz(u2d_Mesh *_mesh);
    u2d_Helmholtz(const u2d_Helmholtz& orig);
    ~u2d_Helmholtz();
    u2d_Problem*run(); /* or solve() ? */
    void setSideNeumann(int side_id);
    void setSideDirichlet(int side_id);
    void setGamma(double _gamma);
    void setAlpha(double _alpha);
    
    /* From already loaded vectors */
    void setSource(dense_vector<double> _source);
    void setAVector(dense_vector<double> _avector);
    void setBVector(dense_vector<double> _bvector);
    void setKVector(dense_vector<double> _kvector);
    void setBField(dense2D<double> _bfield);
    void setVField(dense2D<double> _vfield);
    /* From formulas */
    void setSource(string equation);
    void setAVector(string equation);
    void setBVector(string equation);
    void setKVector(string equation);
    void setBField(string equationx, string equationy);
    void setVField(string equationx, string equationy);
    
    dense2D<double> getDirichletNodes();
    dense2D<double> getBoundaryNodes();
    


    /* Test stuff */
    static void SquareTest();
private:
    /* Classes needed to solve a problem */
    u2d_IsoElement2DP13Points ie2d;
    u2d_IsoElement1DP12Points ie1d;
    u2d_Mesh *mesh;
    u2d_DomainGaussPoints *dgp;
    u2d_BoundaryGaussPoints *bgp;
    u2d_DirichletCondition dc;
    u2d_NeumannCondition nc;
    u2d_LinearProblem<dense2D<double> > *problem;
    u2d_ProblemCalculator *calculator;
    /* Problem parameters */
    double alpha;
    double gamma;
    dense_vector<double> source;
    dense_vector<double> avector;
    dense_vector<double> bvector;
    dense_vector<double> kvector;
    dense2D<double> bfield;
    dense2D<double> vfield;
    /* Problem parameters, equation version*/
    string source_equation;
    string avector_equation;
    string bvector_equation;
    string bfieldx_equation;
    string bfieldy_equation;
    string kvector_equation;
    string vfieldx_equation;
    string vfieldy_equation;
    
    /* To get ready before run()*/
    void calculateVectors();
    

    /* Debug stuff */
    static void SquareTest_debug();
};

#endif	/* _U2D_HELMHOLTZ_H */

