!     
! File:   mesh_project.f90
! Author: Gabriele Dini Ciacci <gab@diniciacci.org> (GDC)
!         Massimo Biava (MB)
!
! Created on May 16, 2011, 10:19 AM
!

!!
! This module stores informations about project file locations.
! @Author Gabriele Dini Ciacci <gab@diniciacci.org>
MODULE mesh_project

    IMPLICIT NONE
    
    CHARACTER (LEN=64), PUBLIC :: project_name
    CHARACTER (LEN=256), PUBLIC  :: project_path
    
    !  #### PUBLIC API METHODS ####
    
    PUBLIC :: skip_comments
    
CONTAINS

    !!
    ! Skips all lines starting with # and moves the position pointer to the
    ! first line starting with a non-# character
    ! @Author Massimo Biava
    SUBROUTINE skip_comments(idf)

        IMPLICIT NONE

        INTEGER, INTENT(IN) :: idf

        CHARACTER(LEN = 1) :: a
        LOGICAL :: ex

        ex = .TRUE.

        DO WHILE (ex)
            READ(idf, '(a1)', END = 10) a
            IF (a .NE. '#') ex = .FALSE.
        ENDDO

        BACKSPACE(idf)

        10 CONTINUE

    END SUBROUTINE skip_comments
    
    !!
    ! Fills module variables with the project_name and project_path from command line
    ! @Author Gabriele Dini Ciacci <gab@diniciacci.org>
    SUBROUTINE init_project()
        
        IMPLICIT NONE

        INTEGER :: nargs;
        
        nargs = COMMAND_ARGUMENT_COUNT() ! This is Fortran 2003
        ! nargs = IARGC() ! This is GNU extension
        
        IF(nargs /= 2) THEN
            WRITE(*,*) 'You must supply only two command line arguments.'
            WRITE(*,*) 'The first must be the project name and the second the project path.'
            WRITE(*,*) 'Project path must not end with a /'
            WRITE(*,*) 'e.g.  umesh2d square /home/foo/bar'
            CALL EXIT ! This is GNU extension
        END IF
        
        ! CALL GETARG(1, project_name) ! This is GNU extension
        ! CALL GETARG(2, project_path) ! This is GNU extension
        CALL GET_COMMAND_ARGUMENT(1, project_name) ! This is Fortran 2003
        CALL GET_COMMAND_ARGUMENT(2, project_path) ! This is Fortran 2003

        
    END SUBROUTINE init_project

END MODULE mesh_project
