#ifndef UMESH2DGUI_H
#define UMESH2DGUI_H

/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
    Copyright (C) 2004 Gabriele Dini Ciacci
    Copyright (C) 2005 Gabriele Dini Ciacci
    Copyright (C) 2006 Gabriele Dini Ciacci
    Copyright (C) 2007 Gabriele Dini Ciacci
    Copyright (C) 2008 Gabriele Dini Ciacci  

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

#include <fltk/Window.h>
#include <fltk/Widget.h>
#include <fltk/run.h>
#include <fltk/Choice.h>
#include <fltk/Color.h>
#include <fltk/Input.h>
#include <fltk/Group.h>
#include <fltk/ValueInput.h>
#include <fltk/Button.h>
#include <fltk/file_chooser.h>
#include <fltk/ask.h> // this is for message, ask, alert...
#include <fltk/ScrollGroup.h>
#include <fltk/MenuBar.h>
#include <fltk/Item.h>
#include <fltk/ItemGroup.h>
#include <fltk/TabGroup.h>
#include <fltk/Box.h>
#include <fltk/Output.h>

#include <stdio.h>

#include "u2d_core.h"
#include "u2d_GL.h"
#include "u2d_GLmesh.h"
#include "u2d_umesh2d_output.h"
#include "u2d_Helmholtz.h"
#include "common/u2d_Project.h"


void debug_cb();

class u2d_Umesh2dgui {
public:
    /* Public general use functions */
    u2d_Umesh2dgui();
    ~u2d_Umesh2dgui();
    void showWindows(int argc, char ** argv);
    void repositionWindows();
    int loop();
    void outputMessage(char *format, ...);

    /* Callbacks and events */
    static void OpenFileBrowserCB(fltk::Widget *widget, void *ptr);
    void openFileBrowserCB(fltk::Widget *widget, void *ptr);
    static void CreationNextButtonCB(fltk::Widget *widget, void *ptr);
    void creationNextButtonCB(fltk::Widget *widget, void *ptr);
    static void ShowLineCB(fltk::Widget *widget, void *ptr);
    void showLineCB(fltk::Widget *widget, void *ptr);
    static void ShowCircleCB(fltk::Widget *widget, void *ptr);
    void showCircleCB(fltk::Widget *widget, void *ptr);
    static void ShowEllipseCB(fltk::Widget *widget, void *ptr);
    void showEllipseCB(fltk::Widget *widget, void *ptr);
    static void ShowDataCB(fltk::Widget *widget, void *ptr);
    void showDataCB(fltk::Widget *widget, void *ptr);
    static void ShowRectangleCB(fltk::Widget *widget, void *ptr);
    void showRectangleCB(fltk::Widget *widget, void *ptr);
    static void NewDataLineCB(fltk::Widget *widget, void *ptr);
    void newDataLineCB(fltk::Widget *widget, void *ptr);
    static void ResetBoundaryTabCB(fltk::Widget *widget, void *ptr);
    void resetBoundaryTabCB(fltk::Widget *widget, void *ptr);
    static void AddLineCB(fltk::Widget *widget, void *ptr);
    void addLineCB(fltk::Widget *widget, void *ptr);
    static void AddCircleCB(fltk::Widget *widget, void *ptr);
    void addCircleCB(fltk::Widget *widget, void *ptr);
    static void AddEllipseCB(fltk::Widget *widget, void *ptr);
    void addEllipseCB(fltk::Widget *widget, void *ptr);
    static void AddDataCB(fltk::Widget *widget, void *ptr);
    void addDataCB(fltk::Widget *widget, void *ptr);
    static void AddRectangleCB(fltk::Widget *widget, void *ptr);
    void addRectangleCB(fltk::Widget *widget, void *ptr);
    static void BoundaryTabNextButtonCB(fltk::Widget *widget, void *ptr);
    void boundaryTabNextButtonCB(fltk::Widget *widget, void *ptr);
    static void DomainTabNextButtonCB(fltk::Widget *widget, void *ptr);
    void domainTabNextButtonCB(fltk::Widget *widget, void *ptr);
    static void DomainTabPrevButtonCB(fltk::Widget *widget, void *ptr);
    void domainTabPrevButtonCB(fltk::Widget *widget, void *ptr);
    static void CreationTabPrevButtonCB(fltk::Widget *widget, void *ptr);
    void creationTabPrevButtonCB(fltk::Widget *widget, void *ptr);
    static void SidesTabNextButtonCB(fltk::Widget *widget, void *ptr);
    void sidesTabNextButtonCB(fltk::Widget *widget, void *ptr);
    static void ParametersTabPrevButtonCB(fltk::Widget *widget, void *ptr);
    void parametersTabPrevButtonCB(fltk::Widget *widget, void *ptr);
    static void SetUmesh2dFormatCB(fltk::Widget *widget, void *ptr);
    void setUmesh2dFormatCB(fltk::Widget *widget, void *ptr);
    static void SetUmesh2dFEFormatCB(fltk::Widget *widget, void *ptr);
    void setUmesh2dFEFormatCB(fltk::Widget *widget, void *ptr);
    static void SetUmesh2dFVFormatCB(fltk::Widget *widget, void *ptr);
    void setUmesh2dFVFormatCB(fltk::Widget *widget, void *ptr);
    static void DeletePrimitiveFromBrowserButtonCB(fltk::Widget *widget, void *ptr);
    void deletePrimitiveFromBrowserButtonCB(fltk::Widget *widget, void *ptr);
    static void RunMainCoreCB(fltk::Widget *widget, void *ptr);
    void runMainCoreCB(fltk::Widget *widget, void *ptr);
    static void AddBackgridLineCB(fltk::Widget *widget, void *ptr);
    void addBackgridLineCB(fltk::Widget *widget, void *ptr);
    static void AddSteinerLineCB(fltk::Widget *widget, void *ptr);
    void addSteinerLineCB(fltk::Widget *widget, void *ptr);
    static void DeleteBackgridLineCB(fltk::Widget *button, void* ptr);
    void deleteBackgridLineCB(fltk::Widget *button, void* ptr);
    static void DeleteSteinerLineCB(fltk::Widget *button, void* ptr);
    void deleteSteinerLineCB(fltk::Widget *button, void* ptr);
    static void SolveProblemCB(fltk::Widget *button, void* ptr);
    void solveProblemCB(fltk::Widget *button, void* ptr);
    static void LoadMeshCB(fltk::Widget *button, void* ptr);
    void loadMeshCB(fltk::Widget *button, void* ptr);
    /* Menu Items Callbacks */
    void openItemCB(fltk::Widget *button, void* ptr);
    static void OpenItemCB(fltk::Widget *button, void* ptr);
    void saveItemCB(fltk::Widget *button, void* ptr);
    static void SaveItemCB(fltk::Widget *button, void* ptr);
    void saveAsItemCB(fltk::Widget *button, void* ptr);
    static void SaveAsItemCB(fltk::Widget *button, void* ptr);
    void quitItemCB(fltk::Widget *button, void* ptr);
    static void QuitItemCB(fltk::Widget *button, void* ptr);
    void aboutItemCB(fltk::Widget *button, void* ptr);
    static void AboutItemCB(fltk::Widget *button, void* ptr);
    void glViewItemCB(fltk::Widget *button, void* ptr);
    static void GlViewItemCB(fltk::Widget *button, void* ptr);
    void browserViewItemCB(fltk::Widget *button, void* ptr);
    static void BrowserViewItemCB(fltk::Widget *button, void* ptr);
    void runHelmholtzSquareTestCB(fltk::Widget *widget, void *ptr);
    static void RunHelmholtzSquareTestCB(fltk::Widget *widget, void *ptr);
    void debugCB(fltk::Widget *widget, void *ptr);
    static void DebugCB(fltk::Widget *widget, void *ptr);


private:
    /* Window creation and destroy */
    void allocateIcons();
    void createMainWindow();
    void createBrowserWindow();
    void createPreviewWindow();
    void createMainWindowTab();
    void createMainWindowBoundaryTab();
    void createMainWindowDomainTab();
    void createMainWindowCreationTab();
    void createMainWindowOutputTab();
    void createMainWindowMeshTab();
    void createMainWindowSidesTab();
    void createMainWindowParametersTab();
    void createMainWindowMenuBar();
    void createMainWindowTitleBox();
    void destroyMainWindow();
    void destroyBrowserWindow();
    void destroyPreviewWindow();
    void destoryIcons();
    
    /* General management functions */
    void setProject(u2d_Project *project_);
    void executeMesher(u2d_Project *project);
    void drawMesh(u2d_Project *project);

    /* Core management functions */
    void addPrimitive(Primitive *prim);

    /* GLOBALS */
    u2d_GL *u2d_gl;
    u2d_GLmesh *u2d_glmesh;
    u2d_Umesh2dOutput *u2d_umesh2d_output;
    u2d_Core *u2d_core;
    /* the tabs */
    fltk::Group *boundarygroup;
    fltk::Group *domaingroup;
    fltk::Group *creationgroup;
    fltk::Group *outputgroup;
    fltk::Group *meshgroup;
    fltk::Group *sidesgroup;
    fltk::Group *parametersgroup;
    /* useful groups */
    fltk::TabGroup *mainwindowtabgroup;
    fltk::ScrollGroup *datagroup;
    fltk::ScrollGroup *steinergroup;
    fltk::ScrollGroup *backgridgroup;
    fltk::MultiBrowser *prim_browser;
    fltk::Group *outcontour_meshinfo;
    fltk::Group *outcontour_sidesinfo;
    fltk::ScrollGroup *sideslistgroup;
    /* the three windows */
    fltk::Window *window;
    fltk::Window *opengl_window;
    fltk::Window *prim_browser_window;
    /* the correspoding useful group subwindows */
    fltk::Group *boundary_subwindow;
    fltk::Group *creation_subwindow;
    fltk::Group *domain_subwindow;
    fltk::Group *sides_subwindow;
    fltk::Group *conditions_subwindow;
    /* the message bar */
    fltk::Output *msgbar;
    /* the text area for shell output */
    fltk::TextDisplay *shell_output;
    /* The menu bar */
    fltk::MenuBar *menubar;
    /* the primitive menubar items */
    fltk::Item *lineitem;
    fltk::Item *circleitem;
    fltk::Item *ellipseitem;
    fltk::Item *dataitem;
    fltk::Item *rectangleitem;
    /* Icons */
    fltk::xpmImage *open_icon;
    fltk::xpmImage *save_icon;
    fltk::xpmImage *create_icon;
    fltk::xpmImage *cancel_icon;
    /* meshinfo Outputs */
    fltk::Output *domainnodesoutput;
    fltk::Output *boundarynodesoutput;
    fltk::Output *domainelementsoutput;
    fltk::Output *boundaryelementsoutput;
    fltk::Output *sidesoutput;
    /* Parameters inputs */
    fltk::Input *avectorinput;
    fltk::Input *bvectorinput;
    fltk::Input *bfieldinput;
    fltk::Input *alphainput;
    fltk::Input *sourcevectorinput;
    fltk::Input *kvectorinput;
    fltk::Input *vxfieldinput;
    fltk::Input *vyfieldinput;
    fltk::Input *gammainput;
    /* A list of radio buttons for vector input */
    std::vector<fltk::Group *> vectorradiogroup;
    /* a terrible list of indexes for primitive values*/
    int inputindex[10];
    int filenameinput_i; //trailing _i is for index
    /* integer that keep track of scroll position */
    int ydataposition;
    int ysteinerposition;
    int ybackgridposition;
    /* A list of radio buttons for sides */
    std::vector<fltk::Group *> sidesradiogroup;
    std::vector<std::stringstream *> sidesradiogroup_labels;
    /* The main project */
    u2d_Project *project;
    /* The mesh object */
    u2d_Mesh *mesh;
    /* For solving the problem */
    u2d_Helmholtz *helmholtz;

};

class Coordinates : public fltk::Group {
    fltk::ValueInput xcoord;
    fltk::ValueInput ycoord;
private:

public:
    Coordinates(int x, int y, int w, int h, const char *label);
    ~Coordinates();

    /*
    double getx(){
      return this->xcoord.value();
    }
    double gety(){
      return this->xcoord.value();
    }
     */

};

Coordinates::Coordinates(int x, int y, int w, int h, const char *label) :
fltk::Group(x, y, w, h, label),
// xcoord(x, y, w, h, "x:"),
//  ycoord(x, y, w, h, "y:") {
xcoord(x + w / 2, y + 2, w / 2 - 1, (h - 4) / 2, "x:"),
ycoord(x + w / 2, y + h - 2 - (h - 4) / 2, w / 2 - 1, (h - 4) / 2, "y:") {

    this->box(fltk::BORDER_FRAME);
    this->align(fltk::ALIGN_INSIDE_LEFT);
    //  xcoord.range(-1000, 1000);
    //  xcoord.step(0.01);
    //  xcoord.linesize(0.01);

    //  ycoord.range(-1000, 1000);
    //  ycoord.step(0.01);
    //  ycoord.linesize(0.01);

    end(); // ends the fltk::Group that automatically starts when instanced.
}

Coordinates::~Coordinates() {


}




#endif
